type 'a t = 'a list;;

let hd = function
 | x :: _ -> x
 | _ -> failwith "Lst.hd: empty list"
;;
let tl = function
 | _ :: xs -> xs
 | _ -> failwith "Lst.tl: empty list"
;;
let rec append xs ys = if xs = [] then ys else hd xs :: append (tl xs) ys;;
let rec init i l = if l <= 0 then [] else i :: init i (l - 1);;
let rec take n xs =
 if n <= 0 then [] else match xs with
  | [] -> []
  | x :: xs -> x :: take (n - 1) xs
;;
let rec drop n xs =
 if n <= 0 then xs else match xs with
  | [] -> []
  | _ :: xs -> drop (n - 1) xs
;;
let split_at n xs = (take n xs, drop n xs);;
let rec fold f b = function
 | [] -> b
 | x :: xs -> f x (fold f b xs)
;;
let rec fold_left f b = function
 | [] -> b
 | x :: xs -> fold_left f (f b x) xs
;;
let rec filter p = function
 | [] -> []
 | x :: xs -> if p x then x :: filter p xs else filter p xs
;;
let length xs = fold (fun _ x -> x + 1) 0 xs;;
let map f xs = fold (fun x xs -> f x :: xs) [] xs;;
let rec last = function
 | [x] -> x
 | _ :: xs -> last xs
 | _ -> failwith "Lst.last: empty list"
;;
let rec nth n = function
 | [] -> failwith "Lst.nth: out of bounds"
 | x :: xs -> if n = 0 then x else nth (n - 1) xs
;;
let rec zip2 xs ys = match (xs, ys) with
 | (x :: xs, y :: ys) -> (x, y) :: zip2 xs ys
 | _ -> []
;;
